@@ -33,7 +33,8 @@ enum {
   PROP_CONSUMER_KEY,
   PROP_CONSUMER_SECRET,
   PROP_TOKEN,
-  PROP_TOKEN_SECRET
+  PROP_TOKEN_SECRET,
+  PROP_SIGNATURE_HOST,
 };
 
 static RestProxyCall *
@@ -67,6 +68,9 @@ oauth_proxy_get_property (GObject *object, guint property_id,
   case PROP_TOKEN_SECRET:
     g_value_set_string (value, priv->token_secret);
     break;
+  case PROP_SIGNATURE_HOST:
+    g_value_set_string (value, priv->signature_host);
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
   }
@@ -99,6 +103,11 @@ oauth_proxy_set_property (GObject *object, guint property_id,
       g_free (priv->token_secret);
     priv->token_secret = g_value_dup_string (value);
     break;
+  case PROP_SIGNATURE_HOST:
+    if (priv->signature_host)
+      g_free (priv->signature_host);
+    priv->signature_host = g_value_dup_string (value);
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
   }
@@ -166,6 +175,13 @@ oauth_proxy_class_init (OAuthProxyClass *klass)
                                    PROP_TOKEN_SECRET,
                                    pspec);
 
+  pspec = g_param_spec_string ("signature-host",  "signature-host",
+                               "The base URL used in the signature string",
+                               NULL, G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class,
+                                   PROP_SIGNATURE_HOST,
+                                   pspec);
+
   /* TODO: add enum property for signature method */
 }
 
@@ -386,6 +402,7 @@ oauth_proxy_request_token (OAuthProxy *proxy,
 
   call = rest_proxy_new_call (REST_PROXY (proxy));
   rest_proxy_call_set_function (call, function ? function : "request_token");
+  rest_proxy_call_set_method (call, "POST");
 
   if (callback_uri)
     rest_proxy_call_add_param (call, "oauth_callback", callback_uri);
@@ -463,6 +480,7 @@ oauth_proxy_request_token_async (OAuthProxy            *proxy,
 
   call = rest_proxy_new_call (REST_PROXY (proxy));
   rest_proxy_call_set_function (call, function ? function : "request_token");
+  rest_proxy_call_set_method (call, "POST");
 
   if (callback_uri)
     rest_proxy_call_add_param (call, "oauth_callback", callback_uri);
@@ -501,6 +519,7 @@ oauth_proxy_access_token (OAuthProxy *proxy,
 
   call = rest_proxy_new_call (REST_PROXY (proxy));
   rest_proxy_call_set_function (call, function ? function : "access_token");
+  rest_proxy_call_set_method (call, "POST");
 
   if (verifier)
     rest_proxy_call_add_param (call, "oauth_verifier", verifier);
@@ -579,6 +598,7 @@ oauth_proxy_access_token_async (OAuthProxy            *proxy,
 
   call = rest_proxy_new_call (REST_PROXY (proxy));
   rest_proxy_call_set_function (call, function ? function : "access_token");
+  rest_proxy_call_set_method (call, "POST");
 
   if (verifier)
     rest_proxy_call_add_param (call, "oauth_verifier", verifier);
@@ -682,6 +702,47 @@ oauth_proxy_is_oauth10a (OAuthProxy *proxy)
   return PROXY_GET_PRIVATE (proxy)->oauth_10a;
 }
 
+/**
+ * oauth_proxy_get_signature_host:
+ * @proxy: an #OAuthProxy
+ *
+ * Get the signature hostname used when creating a signature base string.
+ *
+ * Returns: the signature hostname, or %NULL if there is none set.
+ *  This string is owned by #OAuthProxy and should not be freed.
+ */
+const char *
+oauth_proxy_get_signature_host (OAuthProxy *proxy)
+{
+  OAuthProxyPrivate *priv;
+
+  g_return_val_if_fail (OAUTH_IS_PROXY (proxy), NULL);
+  priv = PROXY_GET_PRIVATE (proxy);
+
+  return priv->signature_host;
+}
+
+/**
+ * oauth_proxy_set_signature_host:
+ * @proxy: an #OAuthProxy
+ * @signature_host: the signature host
+ *
+ * Set the signature hostname used when creating a signature base string.
+ */
+void
+oauth_proxy_set_signature_host (OAuthProxy *proxy,
+                                const char *signature_host)
+{
+  OAuthProxyPrivate *priv;
+
+  g_return_if_fail (OAUTH_IS_PROXY (proxy));
+  priv = PROXY_GET_PRIVATE (proxy);
+
+  g_free (priv->signature_host);
+
+  priv->signature_host = g_strdup (signature_host);
+}
+
 RestProxy *
 oauth_proxy_new_echo_proxy (OAuthProxy  *proxy,
                             /* TODO: should this be a function on the base url? */
